package me.unurled.sacredrealms.sr.commands;

import static me.unurled.sacredrealms.sr.utils.Logger.error;

import io.papermc.paper.command.brigadier.BasicCommand;
import io.papermc.paper.command.brigadier.Commands;
import io.papermc.paper.plugin.bootstrap.BootstrapContext;
import io.papermc.paper.plugin.lifecycle.event.LifecycleEventManager;
import io.papermc.paper.plugin.lifecycle.event.types.LifecycleEvents;
import java.util.Collection;
import java.util.List;
import me.unurled.sacredrealms.sr.commands.admin.AttributeCommand;
import me.unurled.sacredrealms.sr.commands.admin.ClientBuildCommand;
import me.unurled.sacredrealms.sr.commands.admin.CutsceneCommand;
import me.unurled.sacredrealms.sr.commands.admin.EntityTypeCommand;
import me.unurled.sacredrealms.sr.commands.admin.ItemCommand;
import me.unurled.sacredrealms.sr.commands.admin.LevelCommand;
import me.unurled.sacredrealms.sr.commands.admin.SpawnEntityCommand;
import me.unurled.sacredrealms.sr.commands.admin.TreasureCommand;
import me.unurled.sacredrealms.sr.commands.player.DifficultyCommand;
import me.unurled.sacredrealms.sr.commands.player.ResetAdventureCommand;
import me.unurled.srcore.api.Manager;
import org.jetbrains.annotations.NotNull;

public class CommandManager extends Manager {

  private static <T extends BasicCommand> void registerCommand(
      String command, String description, @NotNull BootstrapContext ctx, Class<T> clazz) {
    registerCommand(command, description, List.of(), ctx, clazz);
  }

  private static <T extends BasicCommand> void registerCommand(
      String command,
      String description,
      @NotNull Collection<String> aliases,
      @NotNull BootstrapContext ctx,
      Class<T> clazz) {
    LifecycleEventManager<BootstrapContext> man = ctx.getLifecycleManager();

    try {
      T instance = clazz.getDeclaredConstructor().newInstance();
      man.registerEventHandler(
          LifecycleEvents.COMMANDS,
          event -> {
            final Commands commands = event.registrar();
            commands.register(command, description, aliases, instance);
          });
    } catch (Exception e) {
      error("Failed to register command: " + command + " - " + e.getMessage());
    }
  }

  public static void loadCommands(BootstrapContext ctx) {

    registerCommand("attributes", "Set the attributes of the player.", ctx, AttributeCommand.class);
    registerCommand(
        "clientbuild", "Set the client build of the player.", ctx, ClientBuildCommand.class);
    registerCommand("level", "Set the level of the player.", ctx, LevelCommand.class);
    registerCommand("entitytype", "Create a new entity type.", ctx, EntityTypeCommand.class);
    registerCommand("item", "Create/modify an item.", ctx, ItemCommand.class);
    registerCommand("treasure", " Create a treasure chest.", ctx, TreasureCommand.class);
    registerCommand("spawnentity", "Spawn an entity.", ctx, SpawnEntityCommand.class);

    registerCommand(
        "resetadventure", "Reset the adventure of the player.", ctx, ResetAdventureCommand.class);
    registerCommand(
        "difficulty", "Set the difficulty of the player.", ctx, DifficultyCommand.class);

    registerCommand("cutscene", "Cutscene commands.", List.of("cs"), ctx, CutsceneCommand.class);
  }
}
