package me.unurled.sacredrealms.sr.commands.admin;

import static me.unurled.sacredrealms.sr.utils.Component.textComp;

import io.papermc.paper.command.brigadier.BasicCommand;
import io.papermc.paper.command.brigadier.CommandSourceStack;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import me.unurled.sacredrealms.sr.components.cutscene.Cutscene;
import me.unurled.sacredrealms.sr.components.cutscene.CutsceneManager;
import me.unurled.srcore.api.Manager;
import org.bukkit.Bukkit;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class CutsceneCommand implements BasicCommand {

  private CutsceneManager cutsceneManager;

  /**
   * Returns the permission for the root command used in {@link #canUse(CommandSender)} by default.
   *
   * @return the permission for the root command used in {@link #canUse(CommandSender)}
   */
  @Override
  public @Nullable String permission() {
    return "cutscenes.admin";
  }

  /**
   * Executes the command with the given {@link CommandSourceStack} and arguments.
   *
   * @param sender the commandSourceStack of the command
   * @param args the arguments of the command ignoring repeated spaces
   */
  @Override
  public void execute(@NotNull CommandSourceStack sender, @NotNull String @NotNull [] args) {
    if (cutsceneManager == null) {
      cutsceneManager = Manager.getInstance(CutsceneManager.class);
    }
    // /cs stop, list, reload, play, create, remove
    if (args.length == 1) {
      if (args[0].equals("stop")) {
        if (sender.getSender() instanceof Player p) {
          cutsceneManager.stopCutscene(p);
        } else {
          sender
              .getSender()
              .sendMessage(
                  textComp("<red>You must be a player to stop a cutscene. Use /cs stop <player>"));
        }
      } else if (args[0].equals("list")) {
        sender
            .getSender()
            .sendMessage(
                textComp("<yellow>Cutscenes: " + cutsceneManager.getCutscenesId().toString()));
      } else if (args[0].equals("reload")) {

      }
    } else if (args.length == 2) {
      if (args[0].equals("play")) {
        if (sender.getSender() instanceof Player p) {
          Cutscene cutscene = cutsceneManager.getCutscene(args[1]);
          cutsceneManager.playCutscene(cutscene, p);
        } else {
          sender
              .getSender()
              .sendMessage(
                  textComp(
                      "<red>You must be a player to play a cutscene. Use /cs play <cutscene_id> <player>"));
        }
      } else if (args[0].equals("stop")) {
        Player p = Bukkit.getPlayer(args[1]);
        if (p != null) {
          cutsceneManager.stopCutscene(p);
        }
      } else if (args[0].equals("create")) {
        String name = args[1];
        cutsceneManager.createCutscene(name);
        sender.getSender().sendMessage(textComp("<green>Created cutscene " + name));
      } else if (args[0].equals("remove")) {
        String name = args[1];
        cutsceneManager.removeCutscene(name);
        sender.getSender().sendMessage(textComp("<red>Removed cutscene " + name));
      }
    } else if (args.length == 3) {
      if (args[0].equals("play")) {
        Player p = Bukkit.getPlayer(args[2]);
        if (p != null) {
          Cutscene cutscene = cutsceneManager.getCutscene(args[1]);
          cutsceneManager.playCutscene(cutscene, p);
        }
      }
    } else {
      sender
          .getSender()
          .sendMessage(
              textComp(
                  "<yellow>Usage: /cs <play|stop|reload|list> <cutscene_id|player (only play and stop)> <player (only possible if play with an id before)>"));
    }
  }

  /**
   * Suggests possible completions for the given command {@link CommandSourceStack} and arguments.
   *
   * @param sender the commandSourceStack of the command
   * @param args the arguments of the command including repeated spaces
   * @return a collection of suggestions
   */
  @Override
  public @NotNull Collection<String> suggest(
      @NotNull CommandSourceStack sender, @NotNull String[] args) {
    if (cutsceneManager == null) {
      cutsceneManager = Manager.getInstance(CutsceneManager.class);
    }
    List<String> complete = new ArrayList<>();
    complete.add("stop");
    if (sender.getSender().hasPermission("cutscenes.admin")) {
      complete.add("play");
      complete.add("list");
      complete.add("reload");
      complete.add("create");
      complete.add("remove");
    }
    List<String> result = new ArrayList<>();
    if (args.length == 1) {
      for (String s : complete) {
        if (s.toLowerCase().startsWith(args[0].toLowerCase())) {
          result.add(s);
        }
      }
      return result;
    } else if (args.length == 2) {
      if (sender.getSender().hasPermission("cutscenes.admin")) {
        if (args[0].equalsIgnoreCase("stop")) {
          for (Player p : Bukkit.getOnlinePlayers()) {
            if (p.getName().toLowerCase().startsWith(args[1].toLowerCase())) {
              result.add(p.getName());
            }
          }
        } else if (args[0].equalsIgnoreCase("play") || args[0].equalsIgnoreCase("remove")) {
          for (String s : cutsceneManager.getCutscenesId()) {
            if (s.toLowerCase().startsWith(args[1].toLowerCase())) {
              result.add(s);
            }
          }
        }
      }
      return result;
    } else if (args.length == 3) {
      if (sender.getSender().hasPermission("cutscenes.admin")) {
        if (args[0].equalsIgnoreCase("play")) {
          for (Player p : Bukkit.getOnlinePlayers()) {
            if (p.getName().toLowerCase().startsWith(args[2].toLowerCase())) {
              result.add(p.getName());
            }
          }
        }
      }
    }
    return List.of();
  }
}
