package me.unurled.sacredrealms.sr.components.cutscene;

import com.github.retrooper.packetevents.protocol.entity.pose.EntityPose;
import io.papermc.paper.event.player.PlayerArmSwingEvent;
import me.unurled.sacredrealms.sr.components.cutscene.npc.ArmSwing;
import me.unurled.sacredrealms.sr.components.cutscene.npc.EntityFrame;
import org.bukkit.entity.Player;
import org.bukkit.entity.Pose;
import org.bukkit.event.EventHandler;
import org.bukkit.inventory.EquipmentSlot;
import org.bukkit.inventory.PlayerInventory;
import org.jetbrains.annotations.NotNull;

public class PlayerRecording {
  private final Player player;
  private final EntityFrame entityFrame;
  private ArmSwing swing;

  public PlayerRecording(Player player, EntityFrame entityFrame) {
    this.player = player;
    this.entityFrame = entityFrame;
  }

  private static EntityPose toEntityPose(Pose pose) {
    return pose == Pose.SNEAKING ? EntityPose.SITTING : EntityPose.valueOf(pose.name());
  }

  private static Pose toBukkitPose(EntityPose pose) {
    return pose == EntityPose.SITTING ? Pose.SNEAKING : Pose.valueOf(pose.name());
  }

  @EventHandler
  public void onArmSwing(@NotNull PlayerArmSwingEvent e) {
    if (e.getPlayer().getUniqueId() != player.getUniqueId()) {
      return;
    }
    addSwing(
        e.getHand() == EquipmentSlot.HAND
            ? ArmSwing.RIGHT
            : (e.getHand() == EquipmentSlot.OFF_HAND ? ArmSwing.LEFT : ArmSwing.BOTH));
  }

  private void addSwing(ArmSwing swing) {
    if (this.swing != null && this.swing != swing) {
      this.swing = ArmSwing.BOTH;
      return;
    }

    this.swing = swing;
  }

  public EntityFrame captureFrame() {
    PlayerInventory inv = player.getInventory();
    Pose pose = player.isInsideVehicle() ? Pose.SITTING : player.getPose();
    EntityFrame data =
        new EntityFrame(
            player.getLocation(),
            toEntityPose(pose),
            swing,
            inv.getItemInMainHand().isEmpty() ? null : inv.getItemInMainHand(),
            inv.getItemInOffHand().isEmpty() ? null : inv.getItemInOffHand(),
            inv.getHelmet(),
            inv.getChestplate(),
            inv.getLeggings(),
            inv.getBoots());
    swing = null;
    return data;
  }

  public void applyState(@NotNull EntityFrame value) {
    player.teleport(value.location());
    player.setPose(toBukkitPose(value.pose()));
    PlayerInventory inv = player.getInventory();
    inv.setItemInMainHand(value.mainHand());
    inv.setItemInOffHand(value.offHand());
    inv.setHelmet(value.helmet());
    inv.setChestplate(value.chestplate());
    inv.setLeggings(value.leggings());
    inv.setBoots(value.boots());
    player.updateInventory();
  }
}
