package me.unurled.sacredrealms.sr.components.cutscene.npc;

import com.github.retrooper.packetevents.protocol.entity.pose.EntityPose;
import com.google.gson.JsonElement;
import com.google.gson.JsonPrimitive;
import me.unurled.sacredrealms.sr.utils.Items;
import me.unurled.sacredrealms.sr.utils.gson.LocationSerializer;
import org.bukkit.Location;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

public record EntityFrame(
    Location location,
    EntityPose pose,
    ArmSwing swing,
    ItemStack mainHand,
    ItemStack offHand,
    ItemStack helmet,
    ItemStack chestplate,
    ItemStack leggings,
    ItemStack boots) {

  public static @NotNull EntityFrame deserialize(@NotNull JsonElement jsonElement) {
    Location location =
        new LocationSerializer()
            .deserialize(jsonElement.getAsJsonObject().get("location"), Location.class, null);
    EntityPose pose = EntityPose.valueOf(jsonElement.getAsJsonObject().get("pose").getAsString());
    ArmSwing swing = ArmSwing.valueOf(jsonElement.getAsJsonObject().get("swing").getAsString());
    ItemStack mainHand = Items.deserialize(jsonElement.getAsJsonObject().get("mainHand"));
    ItemStack offHand = Items.deserialize(jsonElement.getAsJsonObject().get("offHand"));
    ItemStack helmet = Items.deserialize(jsonElement.getAsJsonObject().get("helmet"));
    ItemStack chestplate = Items.deserialize(jsonElement.getAsJsonObject().get("chestplate"));
    ItemStack leggings = Items.deserialize(jsonElement.getAsJsonObject().get("leggings"));
    ItemStack boots = Items.deserialize(jsonElement.getAsJsonObject().get("boots"));
    return new EntityFrame(
        location, pose, swing, mainHand, offHand, helmet, chestplate, leggings, boots);
  }

  public @NotNull JsonElement serialize() {
    String s =
        """
        {
          "location": %s,
          "pose": %s,
          "swing": %s,
          "mainHand": %s,
          "offHand": %s,
          "helmet": %s,
          "chestplate": %s,
          "leggings": %s,
          "boots": %s
        }
        """
            .formatted(
                location.serialize(),
                pose.name(),
                swing.name(),
                mainHand.serialize(),
                offHand.serialize(),
                helmet.serialize(),
                chestplate.serialize(),
                leggings.serialize(),
                boots.serialize());
    return new JsonPrimitive(s);
  }
}
