package me.unurled.sacredrealms.sr.components.item;

import java.util.Random;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextColor;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public enum Rarity {
  COMMON("<white>Common", "COMMON", 1d, NamedTextColor.WHITE),
  UNCOMMON("<green>UnCommon", "UnCOMMON", 1_000d, NamedTextColor.GREEN),
  RARE("<dark_aqua>Rare", "RARE", 10_000d, NamedTextColor.DARK_AQUA),
  EPIC("<dark_purple>Epic", "EPIC", 1_000_000d, NamedTextColor.DARK_PURPLE),
  LEGENDARY("<gold>Legendary", "LEGENDARY", 100_000_000d, NamedTextColor.GOLD),
  MYTHIC("<light_purple>Mythic", "MYTHIC", 1_000_000_000d, NamedTextColor.LIGHT_PURPLE),
  SPECIAL("<red>Special", "SPECIAL", 10_000_000_000d, NamedTextColor.RED),
  UNIQUE("<yellow>Unique", "UNIQUE", -1d, NamedTextColor.YELLOW),
  ADMIN(
      "<color:#800000><obf>aa</obf><dark_red>Admin</dark_red><obf>aa</obf></color>",
      "ADMIN",
      -1d,
      NamedTextColor.DARK_RED);

  private final String name;
  private final String id;
  private final Double weight;
  private final TextColor color;

  private static final Random random = new Random();

  Rarity(String name, String id, Double weight, TextColor color) {
    this.name = name;
    this.id = id;
    this.weight = weight;
    this.color = color;
  }

  @Nullable
  public static Rarity getByWeight(int weight) {
    for (Rarity rarity : values()) {
      if (rarity.getWeight() == weight) {
        return rarity;
      }
    }
    return null;
  }

  @NotNull
  public static Rarity getRandomRarity(double luck) {
    // more weight equals less chance to get it
    double randomValue = random.nextDouble() * (1.0 / (luck == 0 ? 1 : luck));

    double cumulativeProbability = 0.0;
    for (Rarity rarity : Rarity.values()) {
      if (rarity.getWeight() == -1) {
        continue;
      }

      cumulativeProbability += 1.0 / rarity.getWeight();

      if (randomValue <= cumulativeProbability) {
        return rarity;
      }
    }
    return Rarity.COMMON;
  }

  public String getName() {
    return name;
  }

  @SuppressWarnings("unused")
  public String getId() {
    return id;
  }

  @SuppressWarnings("unused")
  public Double getWeight() {
    return weight;
  }

  public TextColor getColor() {
    return color;
  }
}
