package me.unurled.sacredrealms.sr.components.treasure;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import java.lang.reflect.Type;
import java.util.HashMap;
import java.util.Map;
import me.unurled.sacredrealms.sr.components.item.ItemStackDeserializer;
import me.unurled.sacredrealms.sr.components.item.ItemStackSerializer;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

public class ItemStackMapSerializer
    implements JsonSerializer<Map<Integer, ItemStack>>,
        JsonDeserializer<Map<Integer, org.bukkit.inventory.ItemStack>> {

  @Override
  public Map<Integer, ItemStack> deserialize(
      @NotNull JsonElement json, Type typeOfT, JsonDeserializationContext context)
      throws JsonParseException {
    Map<Integer, ItemStack> map = new HashMap<>();
    JsonObject jsonObject = json.getAsJsonObject();

    Gson gson =
        new GsonBuilder()
            .registerTypeAdapter(ItemStack.class, new ItemStackDeserializer())
            .create();

    for (Map.Entry<String, JsonElement> entry : jsonObject.entrySet()) {
      Integer key = Integer.parseInt(entry.getKey());
      ItemStack itemStack = gson.fromJson(entry.getValue(), ItemStack.class);
      map.put(key, itemStack);
    }

    return map;
  }

  @Override
  public JsonElement serialize(
      @NotNull Map<Integer, ItemStack> src, Type typeOfSrc, JsonSerializationContext context) {
    JsonObject jsonObject = new JsonObject();

    Gson gson =
        new GsonBuilder().registerTypeAdapter(ItemStack.class, new ItemStackSerializer()).create();

    for (Map.Entry<Integer, ItemStack> entry : src.entrySet()) {
      JsonElement itemStackJson = gson.toJsonTree(entry.getValue(), ItemStack.class);
      jsonObject.add(entry.getKey().toString(), itemStackJson);
    }

    return jsonObject;
  }
}
