package me.unurled.sacredrealms.sr.components.treasure;

import static me.unurled.sacredrealms.sr.utils.Logger.error;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonParseException;
import io.leangen.geantyref.TypeToken;
import java.lang.reflect.Type;
import java.util.Map;
import me.unurled.sacredrealms.sr.components.item.ItemStackDeserializer;
import me.unurled.sacredrealms.sr.utils.gson.LocationSerializer;
import org.bukkit.Location;
import org.bukkit.inventory.ItemStack;

public class TreasureDeserializer implements JsonDeserializer<Treasure> {

  /**
   * Gson invokes this call-back method during deserialization when it encounters a field of the
   * specified type.
   *
   * <p>In the implementation of this call-back method, you should consider invoking {@link
   * JsonDeserializationContext#deserialize(JsonElement, Type)} method to create objects for any
   * non-trivial field of the returned object. However, you should never invoke it on the same type
   * passing {@code json} since that will cause an infinite loop (Gson will call your call-back
   * method again).
   *
   * @param json The Json data being deserialized
   * @param typeOfT The type of the Object to deserialize to
   * @param context
   * @return a deserialized object of the specified type typeOfT which is a subclass of {@code T}
   * @throws JsonParseException if json is not in the expected format of {@code typeofT}
   */
  @Override
  public Treasure deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context)
      throws JsonParseException {
    Gson gson =
        new GsonBuilder()
            .registerTypeAdapter(ItemStack.class, new ItemStackDeserializer())
            .create();

    try {
      Integer id = json.getAsJsonObject().get("id").getAsInt();
      Gson locGson =
          new GsonBuilder()
              .registerTypeAdapter(Location.class, new LocationSerializer())
              .registerTypeAdapter(
                  new TypeToken<Map<Integer, ItemStack>>() {}.getType(),
                  new ItemStackMapSerializer())
              .create();
      Location location = locGson.fromJson(json.getAsJsonObject().get("location"), Location.class);
      String permission = json.getAsJsonObject().get("permission").getAsString();
      String items = json.getAsJsonObject().get("items").getAsString();
      Type hash = new TypeToken<Map<Integer, ItemStack>>() {}.getType();
      Map<Integer, ItemStack> item = gson.fromJson(items, hash);

      if (permission == null || item == null) {
        error("Error deserializing treasure: " + id + " is missing required fields");
        return null;
      }

      return new Treasure(id, location, item, permission);
    } catch (Exception e) {
      error("Error deserializing treasure: " + e.getMessage());
      return null;
    }
  }
}
