package me.unurled.sacredrealms.sr.components.treasure;

import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import me.unurled.sacredrealms.sr.components.item.Item;
import me.unurled.sacredrealms.sr.components.item.ItemManager;
import me.unurled.sacredrealms.sr.components.item.Rarity;
import me.unurled.sacredrealms.sr.components.player.SRPlayer;
import me.unurled.srcore.api.Manager;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;

public class TreasureGenerator {

  public static final int TREASURE_SIZE = 27;
  public static final int MIN_ITEMS = 1;
  private static final int MAX_ITEMS = 10;
  private static ItemManager im;
  private static Random random;

  private TreasureGenerator() {}

  @Contract(pure = true)
  public static @NotNull Treasure generateTreasure(
      @NotNull Treasure treasure, @NotNull SRPlayer p) {
    if (im == null) {
      im = Manager.getInstance(ItemManager.class);
    }
    if (random == null) {
      random = new SecureRandom();
    }
    // generate the treasure based on player level
    int level = p.getLevel();
    double luck = p.getAttribute(Attribute.LUCK);

    // the more level and luck the player has, the better the item he will get
    // the item will have a random rarity
    // max of 10 items
    int nbItems = calculateNumberOfItems(luck, level);

    List<ItemStack> items = new ArrayList<>();

    for (int i = 0; i < nbItems; i++) {
      // generate item
      Rarity rarity = Rarity.getRandomRarity(luck);
      Item item = im.getRandomByRarityItem(rarity);
      ItemStack itemStack = null;
      if (item != null) {
        itemStack = item.toItemStack();
      }
      if (itemStack == null) {
        continue;
      }
      if (rarity.getWeight() < Rarity.EPIC.getWeight()) {
        // cant have more than 1
        itemStack.setAmount(random.nextInt(17));
      }
      // add item to treasure
      items.add(itemStack);
    }
    treasure.setItems(addItemsRandomlyToChest(items));

    return treasure;
  }

  private static @NotNull Map<Integer, ItemStack> addItemsRandomlyToChest(
      @NotNull List<ItemStack> items) {
    List<Integer> availableSlots = new ArrayList<>();
    for (int i = 0; i < TREASURE_SIZE; i++) {
      availableSlots.add(i);
    }

    Collections.shuffle(availableSlots);
    Map<Integer, ItemStack> itemsList = new HashMap<>();
    for (int i = 0; i < items.size(); i++) {
      itemsList.put(availableSlots.get(i), items.get(i));
    }
    return itemsList;
  }

  private static int calculateNumberOfItems(double luck, int level) {
    // Example formula to calculate the number of items based on luck and level
    double modifiedLuck = Math.max(luck, 1); // Ensure luck is at least 1 to avoid division by zero
    int baseItems = MIN_ITEMS + level;
    int maxPossibleItems = (int) Math.min(MAX_ITEMS, baseItems + (modifiedLuck / 10000.0));
    return random.nextInt(maxPossibleItems - MIN_ITEMS + 1) + MIN_ITEMS;
  }
}
