package me.unurled.sacredrealms.sr.data;

import static me.unurled.sacredrealms.sr.utils.Logger.error;

import java.util.List;
import me.unurled.sacredrealms.sr.SR;
import me.unurled.sacredrealms.sr.config.ConfigManager;
import me.unurled.sacredrealms.sr.config.Settings;
import me.unurled.srcore.api.Manager;
import org.jetbrains.annotations.NotNull;
import redis.clients.jedis.JedisPooled;

public class Redis implements DataHandler {

  JedisPooled client;

  public Redis() {
    ConfigManager cm = Manager.getInstance(ConfigManager.class);
    if (cm != null) {
      String host = Settings.REDIS_HOST.toString();
      Integer port = Settings.REDIS_PORT.toInt();
      if (host == null || port == null) {
        error("Failed to get Redis host or port from config, shutting down server.");
        SR.getPlugin().getServer().shutdown();
        return;
      }
      try {
        client = new JedisPooled(host, port);
        client.get("test");
      } catch (Exception e) {
        error("Failed to connect to Redis, shutting down server.");
        SR.getPlugin().getServer().shutdown();
      }
    } else {
      error("Failed to get DataManager instance. Can't connect to Redis, shutting down server.");
      SR.getPlugin().getServer().shutdown();
    }
  }

  /**
   * Get a value from the data source
   *
   * @param key The key to get the value from
   * @return The value of the key
   */
  @Override
  public String get(@NotNull String key) {
    return client.get(key);
  }

  /**
   * Set a value in the data source
   *
   * @param key The key to set the value to
   * @param value The value to set
   */
  @Override
  public void set(@NotNull String key, String value) {
    client.set(key, value);
  }

  /**
   * Remove a value from the data source
   *
   * @param key The key to remove the value from
   */
  @Override
  public void remove(@NotNull String key) {
    client.del(key);
  }

  /**
   * Check if a key exists
   *
   * @param key The key to check
   * @return If the key exists
   */
  @Override
  public boolean exists(@NotNull String key) {
    return client.exists(key);
  }

  /**
   * Get all keys from the data source
   *
   * @param key
   * @return All keys
   */
  @Override
  public List<String> getKeysAll(@NotNull String key) {
    return client.keys(key + "*").stream().toList();
  }
}
