package me.unurled.sacredrealms.sr.utils.character;

import static me.unurled.sacredrealms.sr.utils.Logger.warn;

public class ConfiguredChar {
  private char character;
  private String pngFile;
  private int height;
  private int width;
  private int ascent;

  private ConfiguredChar() {}

  public ConfiguredChar(char character, String pngFile, int height, int width, int ascent) {
    this.character = character;
    this.pngFile = pngFile;
    this.height = height;
    this.width = width;
    this.ascent = ascent;
  }

  public static Builder builder() {
    return new Builder();
  }

  public char getCharacter() {
    return character;
  }

  public String getPngFile() {
    return pngFile;
  }

  public int getHeight() {
    return height;
  }

  public int getWidth() {
    return width;
  }

  public int getAscent() {
    return ascent;
  }

  public String getFile() {
    return pngFile + ".png";
  }

  public static class Builder {

    private final ConfiguredChar configuredChar;

    public Builder() {
      this.configuredChar = new ConfiguredChar();
    }

    public static Builder of() {
      return new Builder();
    }

    public Builder character(char character) {
      configuredChar.character = character;
      return this;
    }

    public Builder png(String png) {
      configuredChar.pngFile = png;
      return this;
    }

    public Builder height(int height) {
      configuredChar.height = height;
      return this;
    }

    public Builder ascent(int ascent) {
      configuredChar.ascent = ascent;
      if (ascent >= configuredChar.height) {
        warn("Invalid config for " + configuredChar.pngFile);
        warn("Ascent " + ascent + " should be no higher than Height " + configuredChar.height);
      }
      return this;
    }

    public Builder descent(int descent) {
      if (descent < 0) {
        warn("Invalid config for " + configuredChar.pngFile);
        warn("Descent " + descent + " should be no lower than 0");
      }
      configuredChar.ascent = configuredChar.height - descent;
      return this;
    }

    public Builder width(int width) {
      configuredChar.width = width;
      return this;
    }

    public ConfiguredChar build() {
      return configuredChar;
    }
  }
}
