package me.unurled.sacredrealms.sr.utils.gson;

import static me.unurled.sacredrealms.sr.utils.Logger.error;

import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonParseException;
import com.google.gson.JsonPrimitive;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import com.mojang.brigadier.exceptions.CommandSyntaxException;
import java.lang.reflect.Type;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.TagParser;
import org.bukkit.Location;
import org.bukkit.craftbukkit.entity.CraftEntity;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.jetbrains.annotations.NotNull;

public class EntityGson {
  public class Deserializer implements JsonDeserializer<Entity> {

    /**
     * Gson invokes this call-back method during deserialization when it encounters a field of the
     * specified type.
     *
     * <p>In the implementation of this call-back method, you should consider invoking {@link
     * JsonDeserializationContext#deserialize(JsonElement, Type)} method to create objects for any
     * non-trivial field of the returned object. However, you should never invoke it on the same
     * type passing {@code json} since that will cause an infinite loop (Gson will call your
     * call-back method again).
     *
     * @param json The Json data being deserialized
     * @param typeOfT The type of the Object to deserialize to
     * @param context
     * @return a deserialized object of the specified type typeOfT which is a subclass of {@code T}
     * @throws JsonParseException if json is not in the expected format of {@code typeofT}
     */
    @Override
    public Entity deserialize(
        @NotNull JsonElement json, Type typeOfT, JsonDeserializationContext context)
        throws JsonParseException {
      // get type from json, get location from json, then create entity and paste nbt data
      EntityType type = EntityType.valueOf(json.getAsJsonObject().get("type").getAsString());
      Location location =
          new LocationSerializer()
              .deserialize(json.getAsJsonObject().get("location"), Location.class, context);
      Entity e = location.getWorld().spawnEntity(location, type);
      net.minecraft.world.entity.Entity nmsEntity = ((CraftEntity) e).getHandle();
      try {
        nmsEntity.load(TagParser.parseTag(json.getAsJsonObject().get("entity").getAsString()));
      } catch (CommandSyntaxException ex) {
        error("Failed to parse entity NBT data " + ex.getMessage());
      }
      e.teleportAsync(location);
      return e;
    }
  }

  public class Serializer implements JsonSerializer<Entity> {

    /**
     * Gson invokes this call-back method during serialization when it encounters a field of the
     * specified type.
     *
     * <p>In the implementation of this call-back method, you should consider invoking {@link
     * JsonSerializationContext#serialize(Object, Type)} method to create JsonElements for any
     * non-trivial field of the {@code src} object. However, you should never invoke it on the
     * {@code src} object itself since that will cause an infinite loop (Gson will call your
     * call-back method again).
     *
     * @param src the object that needs to be converted to Json.
     * @param typeOfSrc the actual type (fully genericized version) of the source object.
     * @param context
     * @return a JsonElement corresponding to the specified object.
     */
    @Override
    public JsonElement serialize(
        @NotNull Entity src, Type typeOfSrc, JsonSerializationContext context) {
      net.minecraft.world.entity.Entity nmsEntity = ((CraftEntity) src).getHandle();
      CompoundTag nbt = new CompoundTag();
      nmsEntity.save(nbt);
      LocationSerializer locationSerializer = new LocationSerializer();
      String s =
          "{\"type\":"
              + src.getType().name()
              + ", \"location\":"
              + locationSerializer.serialize(src.getLocation(), Location.class, context)
              + ", \"entity\":"
              + nbt.getAsString()
              + "}";
      return new JsonPrimitive(s);
    }
  }
}
