package me.unurled.sacredrealms.sr.utils.gson;

import com.google.gson.Gson;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonParseException;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import io.leangen.geantyref.TypeToken;
import java.lang.reflect.Type;
import java.util.Map;
import org.bukkit.Location;
import org.jetbrains.annotations.NotNull;

public class LocationSerializer implements JsonSerializer<Location>, JsonDeserializer<Location> {

  /**
   * Gson invokes this call-back method during serialization when it encounters a field of the
   * specified type.
   *
   * <p>In the implementation of this call-back method, you should consider invoking {@link
   * JsonSerializationContext#serialize(Object, Type)} method to create JsonElements for any
   * non-trivial field of the {@code src} object. However, you should never invoke it on the {@code
   * src} object itself since that will cause an infinite loop (Gson will call your call-back method
   * again).
   *
   * @param src the object that needs to be converted to Json.
   * @param typeOfSrc the actual type (fully genericized version) of the source object.
   * @param context
   * @return a JsonElement corresponding to the specified object.
   */
  @Override
  public JsonElement serialize(
      @NotNull Location src, Type typeOfSrc, JsonSerializationContext context) {
    Gson gson = new Gson();

    return gson.toJsonTree(src.serialize());
  }

  @Override
  public Location deserialize(
      JsonElement jsonElement, Type type, JsonDeserializationContext jsonDeserializationContext)
      throws JsonParseException {
    Gson gson = new Gson();

    Type typ = new TypeToken<Map<String, Object>>() {}.getType();

    return Location.deserialize(gson.fromJson(jsonElement, typ));
  }
}
